/*!
 * merge-deep <https://github.com/jonschlinkert/merge-deep>
 *
 * Copyright (c) 2014-2015, Jon Schlinkert.
 * Licensed under the MIT License.
 */

'use strict';

var cloneDeep = require('clone-deep');
var isObject = require('is-extendable');

module.exports = function mergeDeep(orig, objects) {
  if (!isObject(orig)) {
    orig = {};
  }

  var len = arguments.length;
  var o = cloneDeep(orig);

  for (var i = 1; i < len; i++) {
    merge(o, arguments[i]);
  }
  return o;
};

function merge(o, obj) {
  for (var key in obj) {
    if (!hasOwn(obj, key)) {
      continue;
    }
    var val = obj[key];
    if (isObject(o[key])) {
      o[key] = merge(o[key] || {}, val);
    } else {
      o[key] = cloneDeep(val);
    }
  }
  return o;
}

function hasOwn(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj, key);
}
